# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

"""
This file is meant to be used to test all of the example here and and
in ../packages/services/examples.  We import each of the applications
and add instrument them with a Playwright test that makes sure
there are no console errors or uncaught errors prior to a sentinel
string being printed (see test/example.spec.ts for the sentinel strings
checked before the browser.close() call).
"""
import argparse
import glob
import os.path as osp
import subprocess
import sys
import tempfile

here = osp.abspath(osp.dirname(__file__))


def header(path):
    test_name = osp.basename(path)
    print(
        "\n".join(("\n", "*" * 40, f"Starting {test_name} test in {path}", "*" * 40)),
        flush=True,
    )


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--testPath", help="paths containing this string are tested")
    args = parser.parse_args()

    paths = [i for i in glob.glob("%s/*" % here) if osp.isdir(i)]

    services_dir = osp.abspath(osp.join(here, "../packages/services/examples"))
    paths += list(glob.glob("%s/*" % services_dir))
    if args.testPath:
        paths = [p for p in paths if args.testPath in p]

    print("Testing %s" % paths)
    count = 0
    failed = []
    for path in sorted(paths):
        try:
            if osp.basename(path) == "node":
                with tempfile.TemporaryDirectory() as cwd:
                    header(path)
                    runner = osp.join(path, "main.py")
                    subprocess.check_call([sys.executable, runner], cwd=cwd)  # noqa S603
                    count += 1
            elif osp.exists(osp.join(path, "main.py")):
                with tempfile.TemporaryDirectory() as cwd:
                    header(path)
                    runner = osp.join(here, "example_check.py")
                    subprocess.check_call([sys.executable, runner, path], cwd=cwd)  # noqa S603
                    count += 1
        except subprocess.CalledProcessError:
            failed.append(path)

    if failed:
        msg = "The following examples failed:\n-{}".format("\n-".join(failed))
        raise AssertionError(msg)

    print("\n\n%s tests complete!" % count)


if __name__ == "__main__":
    main()



def option_1():
    """Documentation of 1st option.

    This docstring is intentionally long to fill the documentation panel box.

    Lorem ipsum dolor sit amet, consectetur adipiscing elit. Curabitur vel
    auctor felis. Phasellus non risus maximus tortor molestie consectetur.
    Suspendisse in mauris eget nunc imperdiet elementum. Morbi consequat velit
    at elit suscipit, quis placerat est vulputate. Ut semper leo et fermentum
    gravida. Cras dui nisl, varius et lectus quis, maximus facilisis justo.
    """


def option_2():
    """Documentation of 2nd option."""



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

"""
This file is mean to be called with a path to an example directory as
its argument.  We import the application entry point for the example
and add instrument them with a Playwright test that makes sure
there are no console errors or uncaught errors prior to a sentinel
string being printed.

e.g. python example_check.py ./app
"""
import importlib.util
import os
import shutil
import sys
from pathlib import Path

from jupyterlab.browser_check import run_async_process, run_test
from jupyterlab.labapp import get_app_dir

here = Path(__file__).parent.resolve()
TEST_FILE = here / "example.spec.ts"
REF_SNAPSHOT = Path(TEST_FILE.with_suffix(".ts-snapshots").name) / "example-linux.png"


def main():
    # Load the main file and grab the example class so we can subclass
    example_dir = Path(sys.argv[-1])
    mod_path = (example_dir / "main.py").resolve()
    spec = importlib.util.spec_from_file_location("example", mod_path)
    mod = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(mod)
    sys.modules[__name__] = mod

    class App(mod.ExampleApp):
        """An app that launches an example and waits for it to start up, checking for
        JS console errors, JS errors, and Python logged errors.
        """

        name = __name__
        open_browser = False

        serverapp_config = {
            "base_url": "/foo/",
            "root_dir": str(example_dir.resolve()),
            "preferred_dir": str(example_dir.resolve()),
        }
        ip = "127.0.0.1"

        def initialize_settings(self):
            run_test(self.serverapp, run_browser)
            super().initialize_settings()

        def _jupyter_server_extension_points():
            return [{"module": __name__, "app": App}]

        mod._jupyter_server_extension_points = _jupyter_server_extension_points

    App.__name__ = example_dir.name.capitalize() + "Test"
    App.launch_instance()


async def run_browser(url):
    """Run the browser test and return an exit code."""
    # Run the browser test and return an exit code.
    target = Path(get_app_dir()) / "example_test"
    if not (target / "node_modules").exists():
        if not target.exists():
            target.mkdir(parents=True, exist_ok=True)
        await run_async_process(["npm", "init", "-y"], cwd=str(target))
        await run_async_process(["npm", "install", "-D", "@playwright/test@^1"], cwd=str(target))
        await run_async_process(["npx", "playwright", "install", "chromium"], cwd=str(target))
    test_target = target / TEST_FILE.name

    # Copy test file
    shutil.copy(
        str(TEST_FILE),
        str(test_target),
    )
    # Copy reference snapshot
    example_dir = Path(sys.argv[-1])
    snapshot = example_dir / REF_SNAPSHOT
    has_snapshot = False
    if snapshot.exists():
        has_snapshot = True
        snapshot_target = target / REF_SNAPSHOT
        snapshot_target.parent.mkdir(exist_ok=True)
        shutil.copy(str(snapshot), str(snapshot_target))

    results_target = target / "test-results"
    dst = example_dir / results_target.name
    # Force creation of the results folder as it may be listed in the filebrowser to avoid
    # snapshots discrepancy
    dst.mkdir(exist_ok=True)

    current_env = os.environ.copy()
    current_env["BASE_URL"] = url
    current_env["TEST_SNAPSHOT"] = "1" if has_snapshot else "0"
    try:
        await run_async_process(["npx", "playwright", "test"], env=current_env, cwd=str(target))
    finally:
        # Copy back test-results folder to analyze snapshot error
        if results_target.exists():
            if dst.exists():
                shutil.rmtree(dst)
            shutil.copytree(str(results_target), str(dst))


if __name__ == "__main__":
    main()



"""An INSECURE configuration for Jupyter Server, intended only for mybinder.org."""
# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.
import logging

common = [
    "--no-browser",
    "--debug",
    "--port={port}",
    "--ServerApp.ip=127.0.0.1",
    '--ServerApp.token=""',
    # Disable dns rebinding protection here, since our 'Host' header
    # is not going to be localhost when coming from hub.mybinder.org
    "--ServerApp.allow_remote_access=True",
]


lab_splice_command = " ".join(
    [
        "jupyter",
        "lab",
        "build",
        "--splice-source",
        "--minimize=False",
        "--dev-build=True",
        "--debug",
        ">jupyterlab-spliced.log 2>&1",
        "&&",
        "jupyter",
        "lab",
        "--ServerApp.base_url={base_url}lab-spliced",
        *common,
        ">jupyterlab-spliced.log 2>&1",
    ]
)


c.ServerProxy.servers = {
    "lab-spliced": {
        "command": ["/bin/bash", "-c", lab_splice_command],
        "timeout": 300,
        "absolute_url": True,
    },
}

c.ServerApp.log_level = logging.DEBUG

c.LabApp.dev_mode = True
c.LabApp.extensions_in_dev_mode = True
c.LabApp.skip_dev_build = True

c.ContentsManager.allow_hidden = True



# Copyright (c) 2018 Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

# Generate a GitHub token at https://github.com/settings/tokens
# Invoke this script using something like:
# python scripts/milestone_check.py

import os
import subprocess
import sys

import requests

ranges = {
    "0.35": "origin/0.35.0 --not origin/0.34.x",
    "0.35.x": "origin/0.35.x --not v0.35.0",
    "1.0": "origin/1.0.x --not origin/0.35.x",
    "1.1": "v1.1.0 --not origin/1.0.x",
    "1.1.1": "v1.1.1 --not v1.1.0",
    "1.1.2": "v1.1.2 --not v1.1.1",
    "1.1.3": "v1.1.3 --not v1.1.2",
    "1.2": "origin/1.x --not origin/1.1.x",
    "2.0": "v2.0.0 --not origin/1.x",
    "2.0.1": "v2.0.1 --not v2.0.0",
    "2.0.2": "origin/2.0.x --not v2.0.1",
    "2.1": "origin/2.1.x --not origin/2.0.x",
    "2.2": "origin/2.2.x --not origin/2.1.x",
    # 6507205805 is a commit in the debugger ancestor tree before merging
    "3.0": "origin/main ^origin/2.2.x ^6507205805",
}

try:
    api_token = os.environ["GITHUB_TOKEN"]
except KeyError:
    print(
        "Error: set the environment variable GITHUB_TOKEN to a GitHub authentication token (see https://github.com/settings/tokens)"
    )
    sys.exit(1)

if len(sys.argv) != 2:  # noqa
    print("Error: exactly one argument expected, the milestone.")
    sys.exit(1)

MILESTONE = sys.argv[1]

if MILESTONE not in ranges:
    print(
        f"Error: I do not know about milestone {MILESTONE!r}. Possible milestones are {list(ranges.keys())!r}"
    )
    sys.exit(1)


out = subprocess.run(
    f"git log {ranges[MILESTONE]} --format='%H,%cE,%s'",
    shell=True,  # noqa S602
    encoding="utf8",
    stdout=subprocess.PIPE,
    check=True,
)
commits = {i[0]: (i[1], i[2]) for i in (x.split(",", 2) for x in out.stdout.splitlines())}


url = "https://api.github.com/graphql"
json = {
    "query": """
query test($cursor: String) {
  search(first: 50, after: $cursor, type: ISSUE, query: "repo:jupyterlab/jupyterlab milestone:%s is:pr is:merged ") {
    issueCount
    pageInfo {
      endCursor
      hasNextPage
    }
    nodes {
      ... on PullRequest {
        title
        number
        mergeCommit {
          oid
        }
        commits(first: 100) {
          totalCount
          nodes {
            commit {
              oid
            }
          }
        }
      }
    }
  }
}
"""
    % MILESTONE,
    "variables": {"cursor": None},
}


headers = {"Authorization": "token %s" % api_token}
# construct a commit to PR dictionary
prs = {}
large_prs = []
cursor = None
while True:
    json["variables"]["cursor"] = cursor
    r = requests.post(url=url, json=json, headers=headers, timeout=120)
    results = r.json()["data"]["search"]
    total_prs = results["issueCount"]

    pr_list = results["nodes"]
    for pr in pr_list:
        if pr["commits"]["totalCount"] > 100:  # noqa
            large_prs.append(pr["number"])
            continue
            # TODO fetch commits
        prs[pr["number"]] = {
            "mergeCommit": pr["mergeCommit"]["oid"],
            "commits": {i["commit"]["oid"] for i in pr["commits"]["nodes"]},
        }

    has_next_page = results["pageInfo"]["hasNextPage"]
    cursor = results["pageInfo"]["endCursor"]

    if not has_next_page:
        break

prjson = {
    "query": """
query test($pr:Int!, $cursor: String) {
  repository(owner: "jupyterlab", name: "jupyterlab") {
    pullRequest(number: $pr) {
      title
      number
      mergeCommit {
        oid
      }
      commits(first: 100, after: $cursor) {
        totalCount
        pageInfo {
          endCursor
          hasNextPage
        }
        nodes {
          commit {
            oid
          }
        }
      }
    }
  }
}
""",
    "variables": {"pr": None, "cursor": None},
}

for prnumber in large_prs:
    prjson["variables"]["pr"] = prnumber
    pr_commits = set()
    while True:
        r = requests.post(url=url, json=prjson, headers=headers, timeout=120)
        pr = r.json()["data"]["repository"]["pullRequest"]
        assert pr["number"] == prnumber  # noqa
        total_commits = pr["commits"]["totalCount"]
        pr_commits.update(i["commit"]["oid"] for i in pr["commits"]["nodes"])
        has_next_page = results["pageInfo"]["hasNextPage"]
        cursor = results["pageInfo"]["endCursor"]

        if not pr["commits"]["pageInfo"]["hasNextPage"]:
            break
        prjson["variables"]["cursor"] = pr["commits"]["pageInfo"]["endCursor"]

    prs[prnumber] = {"mergeCommit": pr["mergeCommit"]["oid"], "commits": pr_commits}
    if total_commits > len(pr_commits):
        print(
            "WARNING: PR %d (merge %s) has %d commits, but GitHub is only giving us %d of them"
            % (prnumber, pr["mergeCommit"]["oid"], total_commits, len(pr_commits))
        )


# Check we got all PRs
assert len(prs) == total_prs  # noqa

# Reverse dictionary
commits_to_prs = {}
for key, value in prs.items():
    commits_to_prs[value["mergeCommit"]] = key
    for c in value["commits"]:
        commits_to_prs[c] = key

# Check to see if commits in the repo are represented in PRs
good = set()
notfound = set()
for c in commits:
    if c in commits_to_prs:
        good.add(commits_to_prs[c])
    else:
        notfound.add(c)

prs_not_represented = set(prs.keys()) - good

print("Milestone: %s, %d merged PRs, %d commits in history" % (MILESTONE, total_prs, len(commits)))

print()
print("-" * 40)
print()

if len(prs_not_represented) > 0:
    print(
        """
PRs that are in the milestone, but have no commits in the version range.
These PRs probably belong in a different milestone.
"""
    )
    print(
        "\n".join(
            "https://github.com/jupyterlab/jupyterlab/pull/%d" % i for i in prs_not_represented
        )
    )
else:
    print(
        "Congratulations! All PRs in this milestone have commits in the commit history for this version range, so they all probably belong in this milestone."
    )

print()
print("-" * 40)
print()

if len(notfound):
    print(
        """The following commits are not included in any PR on this milestone.
This probably means the commit's PR needs to be assigned to this milestone,
or the commit was pushed to main directly.
"""
    )
    print("\n".join(f"{c} {commits[c][0]} {commits[c][1]}" for c in notfound))
    prs_to_check = [
        c
        for c in notfound
        if "Merge pull request #" in commits[c][1] and commits[c][0] == "noreply@github.com"
    ]
    if len(prs_to_check) > 0:
        print()
        print(
            "Try checking these PRs. They probably should be in the milestone, but probably aren't:"
        )
        print()
        print("\n".join(f"{c} {commits[c][1]}" for c in prs_to_check))
else:
    print(
        "Congratulations! All commits in the commit history are included in some PR in this milestone."
    )



# Copyright (c) Jupyter Development Team.
# Distributed under the terms of the Modified BSD License.

import os

HERE = os.path.dirname(__file__)

os.environ["JUPYTERLAB_SETTINGS_DIR"] = str(os.path.join(HERE, "settings"))

import json  # noqa

from jupyter_server.base.handlers import FileFindHandler  # noqa
from jupyter_server.utils import url_path_join as ujoin  # noqa
from traitlets import Unicode  # noqa

from jupyterlab.labapp import LabApp  # noqa

with open(os.path.join(HERE, "package.json")) as fid:
    version = json.load(fid)["version"]


class ListingsApp(LabApp):
    base_url = "/"
    default_url = Unicode("/lab", help="The default URL to redirect to from `/`")

    def init_webapp(self):
        """initialize tornado webapp and httpserver."""
        super().init_webapp()
        default_handlers = [
            (
                ujoin(self.base_url, r"/listings/(.*)"),
                FileFindHandler,
                {"path": os.path.join(HERE, "list")},
            )
        ]
        self.web_app.add_handlers(".*$", default_handlers)


if __name__ == "__main__":
    ListingsApp.launch_instance()
